HoraceAbilities = HoraceAbilities or {}

HoraceAbilities.CorpsesToCheckForMimic = HoraceAbilities.CorpsesToCheckForMimic or {}

function HoraceAbilities.OnPlayerUpdate(Player)
    local playerData = Player:getModData()
    local now = getTimestamp()
    local lastSlurp = playerData.lastSlurp or 0
    if playerData.gettingSlurped then
        if now - lastSlurp > 5 or playerData.cancelSlurp then
            playerData.gettingSlurped = false
            playerData.cancelSlurp = false
        else
            Player:setSlowFactor(0.9)
            Player:setSlowTimer(500)
            Player:setSneaking(true)
        end 
    end

    for i,corpse in ipairs(HoraceAbilities.CorpsesToCheckForMimic) do
        local objectId = corpse:getObjectID()
        if objectId ~= -1 then
            local horace = HoraceTypes.GetCorpseHorace(corpse)
            if horace and horace.Mimic and horace.Mimic.Active then
                sendClientCommand(Player, "horace", "UnmimicCorpse", {corpseId = objectId, x = corpse:getX(), y = corpse:getY(), z = corpse:getZ(), slot = horace.Mimic.InactiveMimicSlot, item = horace.Mimic.InactiveMimicItem})
                HoraceAbilities.CorpsesToCheckForMimic[i] = nil
            end
        end
    end
end

function HoraceAbilities.OnHoraceUpdate(Zombie, Horace, HoraceData)
    -- TODO some things maybe only if not remote zombie?
    local now = getTimestamp()

    local target = Zombie:getTarget()
    if Horace.BumpOnCharge and target then
        local playerData = target:getModData()
        local lastBumpedOnCharge = playerData.LastBumpedOnCharge or 0
        if target and Zombie:isZombieAttacking(target) and now - lastBumpedOnCharge > Horace.BumpOnCharge.Cooldown and not Zombie:isCrawling() then
            if Horace.Slurping then
                Zombie:setForceEatingAnimation(true)
                Zombie:faceThisObject(target)
            end

            target:setBumpType("left");
            target:setBumpFallType("pushedFront"); -- TODO maybe check if zombie behind or front
            target:setVariable("BumpDone", false);
            target:setVariable("BumpFall", true);
            target:setVariable("TripObstacleType", "zombie");

            playerData.LastBumpedOnCharge = now
        end
    end

    if Zombie:isLocal() then
        if Horace.InjuryInflicting and target then
            local zombieAttacking = Zombie:isZombieAttacking(target)
            local playerHitReacting = target:isCurrentState(PlayerHitReactionState.instance())
            if zombieAttacking and playerHitReacting and not HoraceData.InflictingInjury then
                HoraceData.InflictingInjury = true

                local bodyDamage = target:getBodyDamage()
                local injuryCount = HoraceAbilities.CountInjuries(bodyDamage, Horace.InjuryInflicting.Injury)
                local partCandidates = Horace.InjuryInflicting.OrderedPartChoices[injuryCount + 1] or Horace.InjuryInflicting.OrderedPartChoices[#Horace.InjuryInflicting.OrderedPartChoices]
                local bodyPart = nil
                for i=1,3 do
                    local partChoice = partCandidates[ZombRand(#partCandidates) + 1]
                    local part = bodyDamage:getBodyPart(BodyPartType[partChoice])
                    if not part:isDeepWounded() and part:getFractureTime() <= 0 then
                        bodyPart = part
                        break
                    end
                end
                if bodyPart then
                    if Horace.InjuryInflicting.Injury == "Fracture" then
                        bodyPart:setFractureTime(10 + ZombRand(20))
                    elseif Horace.InjuryInflicting.Injury == "DeepWound" then
                        bodyPart:setDeepWounded(true)
                        bodyPart:setDeepWoundTime(10 + ZombRand(20))
                        bodyPart:setBleedingTime(10 + ZombRand(20))
                    end
                end
            elseif not zombieAttacking and not playerHitReacting and HoraceData.InflictingInjury then
                HoraceData.InflictingInjury = false
            end
        end
    end

    if Horace.AvoidCrawling and Zombie:isCrawling() then
        Zombie:setBecomeCrawler(false)
        Zombie:setCrawler(false)
    end

    if Zombie:isLocal() then
        local retreatedAt = HoraceData.RetreatedAt or 0
        if HoraceData.AboutToRetreat then
            local target = Zombie:getTarget()
            if target then
                HoraceData.RetreatedFrom = target:getUsername()
            end

            if retreatedAt > 0 then
                if Zombie:isMoving() then
                    Zombie:getZombieLungeSpeed()
                    HoraceData.AboutToRetreat = false
                end
            end

            local minRange = Horace.Retreating.MinRange
            local maxRange = Horace.Retreating.MaxRange
            local dirX = ZombRand(2) == 0 and -1 or 1
            local dirY = ZombRand(2) == 0 and -1 or 1
            local x = Zombie:getX() + (minRange + ZombRand(maxRange - minRange)) * dirX
            local y = Zombie:getY() + (minRange + ZombRand(maxRange - minRange)) * dirY

            Zombie:getPathFindBehavior2():pathToLocation(x, y, Zombie:getZ())
            Zombie:setVariable("bPathfind", true)
            Zombie:setMoving(true)

            Zombie:setUseless(true)
            HoraceData.RetreatedAt = now
        end

        if retreatedAt > 0 then
            local timeSpentRetreating = Horace.Retreating and (Horace.Retreating.TimeSpentRetreating or 5) or 5
            if timeSpentRetreating then
                if now - retreatedAt > timeSpentRetreating then -- TODO reset once they reach their target too
                    Zombie:setUseless(false)
                    HoraceData.RetreatedAt = nil
                    if Horace.Retreating.ReturnAfter and HoraceData.RetreatedFrom then
                        local restoreTarget = getPlayerFromUsername(HoraceData.RetreatedFrom)
                        if restoreTarget then
                            Zombie:setTarget(restoreTarget)
                            Zombie:pathToCharacter(restoreTarget)
                            Zombie:getZombieLungeSpeed()
                        end
                        HoraceData.RetreatedFrom = nil
                    end
                end
            end
        end
    end

    -- Prevent reused zombies from being stuck useless
    if Zombie:isUseless() and not Horace.Retreating or not HoraceData.RetreatedAt then
        Zombie:setUseless(false)
    end

    if Zombie:isLocal() then
        if Horace.Screeching then
            if Horace.Screeching.NeverAttacks and target then
                Zombie:setUseless(true)
            end

            local lastScreechedAt = HoraceData.LastScreechedAt or 0
            if HoraceData.ScreechingScheduledAt then
                if now >= HoraceData.ScreechingScheduledAt then
                    sendClientCommand(getPlayer(), "horace", "Screech", { zombieId = Zombie:getOnlineID(), sound = Horace.Screeching.Sound, x = Zombie:getX(), y = Zombie:getY(), z = Zombie:getZ(), message = Horace.Screeching.ScreechMessage })
                    getWorldSoundManager():addSound(nil, Zombie:getX(), Zombie:getY(), Zombie:getZ(), Horace.Screeching.SoundVolume or 60, Horace.Screeching.SoundRange or 60, false)
                    HoraceData.LastScreechedAt = now
                    HoraceData.ScreechingScheduledAt = nil
                    HoraceData.ScreechingEndsAt = now + (Horace.Screeching.ScreechTime or 10)
                else
                    Zombie:setVariable("onknees", true)
                end
            elseif HoraceData.ScreechingEndsAt then
                Zombie:setVariable("onknees", false)
                if now >= HoraceData.ScreechingEndsAt then
                    Zombie:setForceEatingAnimation(false)
                    if Horace.Screeching.RetreatAfter then
                        HoraceData.AboutToRetreat = true
                    end
                end
            elseif now - lastScreechedAt > Horace.Screeching.Cooldown then
                local target = Zombie:getTarget()
                if target then
                    sendClientCommand(getPlayer(), "horace", "ScreechInhale", { zombieId = Zombie:getOnlineID(), sound = Horace.Screeching.PreparationSound, x = Zombie:getX(), y = Zombie:getY(), z = Zombie:getZ(), message = Horace.Screeching.WarnMessage })
                    Zombie:setForceEatingAnimation(true)
                    Zombie:setVariable("onknees", true)
                    HoraceData.ScreechingScheduledAt = now + (Horace.Screeching.ChargeTime or 10)
                end
            end
        end

        if Horace.Slurping then
            local target = Zombie:getTarget()
            if target then
                local playerData = target:getModData()
                if target:isBumped() or target:isKnockedDown() then
                    playerData.gettingSlurped = true
                    playerData.slurpX = target:getX()
                    playerData.slurpY = target:getY()
                    HoraceData.slurpTarget = target:getUsername()
                    playerData.lastSlurp = now
                    HoraceData.lastSlurp = 0
                elseif HoraceData.slurpTarget then
                    if Zombie:getDistanceSq(target) > 1 then
                        Zombie:setForceEatingAnimation(false)
                        HoraceData.slurpTarget = nil
                        playerData.LastBumpedOnCharge = 0
                        playerData.cancelSlurp = true
                    else
                        Zombie:faceLocationF(target:getX() + target:getForwardDirection():getX() * 0.1, target:getY() + target:getForwardDirection():getY() * 0.1)
                        if Horace.Slurping.AvoidKneeling then
                            Zombie:setVariable("onknees", false)
                        end
                        if ZombRand(60) == 0 then
                            Zombie:splatBloodFloor()
                            target:splatBloodFloor()
                        end

                        local lastSlurp = HoraceData.lastSlurp or 0
                        if now - lastSlurp > Horace.Slurping.Cooldown then
                            sendClientCommand(target, "horace", "Slurp", { zombieId = Zombie:getOnlineID(), sound = Horace.Slurping.Sound, x = Zombie:getX(), y = Zombie:getY(), z = Zombie:getZ(), message = string.gsub("*latches onto %target%'s neck*", "%%target%%", target:getDescriptor():getForename()) })
                            target:splatBloodFloorBig()
                            Zombie:playBloodSplatterSound()
                            local minDmgPerSlurp = 0.1
                            local maxDmgPerSlurp = 0.2

                            target:getBodyDamage():SetBitten(BodyPartType.Neck, true)
                            target:getBodyDamage():SetBleeding(BodyPartType.Neck, true)
                            target:getBodyDamage():AddDamage(BodyPartType.Neck, ZombRandFloat(minDmgPerSlurp,maxDmgPerSlurp))
        
                            playerData.lastSlurp = now
                            HoraceData.lastSlurp = now
                            playerData.gettingSlurped = true
                        end
                    end
                else
                    Zombie:setForceEatingAnimation(false)
                end
            else
                Zombie:setForceEatingAnimation(false)
            end
        end

        if Horace.Mimic then
            if Horace.Mimic.MimicOnTarget then
                local target = Zombie:getTarget()
                if target then
                    if HoraceData.MimicTarget ~= target:getUsername() then
                        HoraceAbilities.StartMimicking(Zombie, target, Horace, HoraceData)
                    end
                end
            elseif Horace.Mimic.MimicOnCharge then
                local target = Zombie:getTarget()
                if target and (Zombie:isCurrentState(WalkTowardState.instance()) or Zombie:isCurrentState(LungeState.instance())) then
                    if HoraceData.MimicTarget ~= target:getUsername() then
                        HoraceAbilities.StartMimicking(Zombie, target, Horace, HoraceData)
                        sendClientCommand(target, "horace", "Mimic", { zombieId = Zombie:getOnlineID(), x = Zombie:getX(), y = Zombie:getY(), z = Zombie:getZ(), target = target:getUsername() })
                    end
                end
            end
        end
    end
end

function HoraceAbilities.OnHoraceHit(Zombie, Player, BodyPart, Weapon, Horace, HoraceData)
    local now = getTimestamp()

    local playerData = Player:getModData()
    if playerData.gettingSlurped then -- TODO technically should check if this is the slurping zombie
        if ZombRand(5) == 0 then -- TODO make property
            Zombie:setForceEatingAnimation(false)
            playerData.cancelSlurp = true
        else
            Player:setKnockedDown(true)
        end
    end

    if Horace.AvoidDamageChance then
        if not Player:isDoShove() and ZombRandFloat(0, 1) <= Horace.AvoidDamageChance then
            Zombie:setStaggerTimeMod(Weapon:getPushBackMod() * Weapon:getKnockbackMod(Zombie) * Player:getShovingMod())
            Zombie:setAvoidDamage(true)
        end
    end

    if Horace.Retreating then
        if (not Horace.Retreating.LimitOnce or not HoraceData.RetreatedOnce) and ZombRandFloat(0, 1) <= Horace.Retreating.Chance then
            if Horace.Retreating.RetreatResetsBumpOnCharge then
                playerData.LastBumpedOnCharge = 0
            end
            HoraceData.AboutToRetreat = true
            HoraceData.RetreatedOnce = true
        end

        if Zombie:isUseless() and Horace.Retreating.ResetOnHit then
            HoraceData.RetreatedOnce = false
            Zombie:setUseless(false)
        end
    end

    if Horace.BumpOnCharge and Horace.BumpOnCharge.CooldownOnParry then
        HoraceData.LastBumpedOnCharge = getTimestamp()
    end

    if Horace.RollOnGround then
        if Zombie:isOnFloor() and ZombRandFloat(0, 1) <= Horace.RollOnGround.Chance then
            Zombie:toggleCrawling()
        end
    end

    if Horace.TripStompers then
        if Zombie:isOnFloor() and ZombRandFloat(0, 1) <= Horace.TripStompers.Chance then
            Zombie:toggleCrawling()
            
            Player:setBumpType("left");
            Player:setVariable("BumpDone", false);
            Player:setVariable("BumpFall", true);
            Player:setVariable("TripObstacleType", "zombie");
        end
    end

    if Horace.Mimic and Horace.Mimic.MimicOnHit then
        if HoraceData.MimicTarget ~= Player:getUsername() then
            HoraceAbilities.StartMimicking(Zombie, Player, Horace, HoraceData)
        end
    end

    if Horace.Screeching and Horace.Screeching.ResetsOnHit then
        Zombie:setForceEatingAnimation(false)
        HoraceData.ScreechingScheduledAt = nil
    end
end

function HoraceAbilities.OnHoraceDead(Zombie, Horace, HoraceData)
    HoraceTypes.ResetHoraceData(HoraceData)

    if Horace.Slurping then
        local target = Zombie:getTarget()
        if target then
            local playerData = target:getModData()
            playerData.cancelSlurp = true
        end
    end
end

function HoraceAbilities.StartMimicking(Zombie, Player, Horace, HoraceData)
    local wornItems = Zombie:getWornItems()
    wornItems:clear()
    wornItems:copyFrom(Player:getWornItems())
    wornItems:setItem("Neck", InventoryItemFactory.CreateItem("Base.MimicActive"))
    wornItems:getItemVisuals(Zombie:getItemVisuals())

    local zombieVisual = Zombie:getHumanVisual()
    local playerVisual = Player:getHumanVisual()
    Zombie:setFemale(Player:isFemale())
    zombieVisual:setSkinColor(playerVisual:getSkinColor())
    zombieVisual:setHairColor(playerVisual:getHairColor())
    zombieVisual:setHairModel(playerVisual:getHairModel())
    zombieVisual:setBeardModel(playerVisual:getBeardModel())
    zombieVisual:setBeardColor(playerVisual:getBeardColor())
    zombieVisual:setSkinTextureIndex(playerVisual:getSkinTextureIndex())
    zombieVisual:setBodyHairIndex(playerVisual:getBodyHairIndex())
    Zombie:addRandomBloodDirtHolesEtc()
    HoraceData.MimicTarget = Player:getUsername()
    Zombie:resetModelNextFrame()
end

function HoraceAbilities.StopMimicking(Zombie)
    local wornItems = Zombie:getWornItems()
    wornItems:clear()
    wornItems:setItem("FullSuit", InventoryItemFactory.CreateItem("Base.Mimic"))
    wornItems:getItemVisuals(Zombie:getItemVisuals())
    Zombie:resetModelNextFrame()
end

function HoraceAbilities.OnContainerUpdate(Container)
    if Container and Container.getObjectID then
        table.insert(HoraceAbilities.CorpsesToCheckForMimic, Container)
    end
end

function HoraceAbilities.CountInjuries(BodyDamage, Injury)
    local count = 0
    for _, bodyPart in ipairs(BodyDamage:getBodyParts()) do
        if Injury == "DeepWound" and bodyPart:deepWounded() then
            count = count + 1
        elseif Injury == "Fracture" and bodyPart:getFractureTime() > 0 then
            count = count + 1
        end
    end
    return count
end

Events.OnPlayerUpdate.Add(HoraceAbilities.OnPlayerUpdate)
Events.OnContainerUpdate.Add(HoraceAbilities.OnContainerUpdate)