HoraceTypes = HoraceTypes or {}

HoraceTypes.HoraceTypes = HoraceTypes.HoraceTypes or {}
HoraceTypes.HoraceSlotActivations = HoraceTypes.HoraceSlotActivations or {}
HoraceTypes.HoraceItemActivations = HoraceTypes.HoraceItemActivations or {}

-- Sound Options (will pick randomly if table):
-- * IdleSound
-- * HitSound
-- * DeathSound
-- * TargetSound
-- * ChargeSound (NYI)

-- Abilities:
-- * BumpOnCharge (Cooldown)

-- Other Properties:
-- * Name
-- * AvoidCrawling
-- * VehicleCrashed
-- * VehicleKilled

function HoraceTypes.Register(Name, ActivationSlot, ActivationItem, HoraceDefinition)
    HoraceTypes.HoraceTypes[Name] = HoraceDefinition
    HoraceDefinition.Name = Name
    HoraceTypes.HoraceSlotActivations[ActivationSlot] = HoraceTypes.HoraceSlotActivations[ActivationSlot] or {}
    HoraceTypes.HoraceSlotActivations[ActivationSlot][ActivationItem] = HoraceDefinition
    HoraceTypes.HoraceItemActivations[ActivationItem] = HoraceDefinition
end

function HoraceTypes.GetPlayerHorace(Player)
    for slot, expectedItems in pairs(HoraceTypes.HoraceSlotActivations) do
        for expectedItem, horace in pairs(expectedItems) do
            local equippedItem = Player:getWornItems():getItem(slot)
            if equippedItem and equippedItem:getVisual():getItemType() == expectedItem then
                return horace
            end
        end
    end
    return nil
end

function HoraceTypes.GetZombieHorace(Zombie)
    for i = 0, Zombie:getItemVisuals():size() - 1 do
        local itemName = Zombie:getItemVisuals():get(i):getItemType()
        local horace = HoraceTypes.HoraceItemActivations[itemName]
        if horace then
            return horace
        end
    end
    return nil
end

function HoraceTypes.GetCorpseHorace(Corpse)
    for slot, expectedItems in pairs(HoraceTypes.HoraceSlotActivations) do
        for expectedItem, horace in pairs(expectedItems) do
            local equippedItem = Corpse:getWornItems():getItem(slot)
            if equippedItem and equippedItem:getVisual():getItemType() == expectedItem then
                return horace
            end
        end
    end
    return nil
end

function HoraceTypes.PickSound(Sounds)
    if type(Sounds) == "table" then
        return Sounds[ZombRand(#Sounds) + 1]
    else
        return Sounds
    end
end

function HoraceTypes.ResetHoraceData(HoraceData)
    HoraceData.OnlineID = nil
    HoraceData.MimicTarget = nil
    HoraceData.LastTarget = nil
    HoraceData.LastTargetCharge = nil
    HoraceData.RetreatedAt = nil
    HoraceData.AboutToRetreat = nil
    HoraceData.RetreatedFrom = nil
    HoraceData.LastScreechedAt = nil
    HoraceData.ScreechingScheduledAt = nil
    HoraceData.ScreechingEndsAt = nil
    HoraceData.RetreatedOnce = nil
    HoraceData.lastVehicleHitForce = nil
    HoraceData.InflictingInjury = nil
end

function HoraceTypes.GetSaneHoraceData(Zombie)
    local horaceData = Zombie:getModData()
    local onlineId = Zombie:getOnlineID()
    if horaceData.OnlineID and onlineId ~= horaceData.OnlineID then
        HoraceTypes.ResetHoraceData(horaceData)
    end
    horaceData.OnlineID = onlineId
    return horaceData
end