HoraceVehicleDamage = HoraceVehicleDamage or {}

function HoraceVehicleDamage.DetectHits(Zombie, Horace, HoraceData)
    local hitForce = Zombie:getHitForce()
    local lastVehicleHitForce = HoraceData.lastVehicleHitForce or hitForce
    if hitForce ~= lastVehicleHitForce then
        local attacker = Zombie:getAttackedBy()
        if attacker and attacker:isDriving() and attacker:getDistanceSq(Zombie) < 10 then
            attacker:setHaloNote(Horace.Name .. " hit force: " .. hitForce, 255, 0, 0, 300)
            local vehicle = attacker:getVehicle()
            sendClientCommand(attacker, 'horace', 'HoraceCrash', { vehicle = vehicle:getId(), hitForce = hitForce, horace = Horace.Name, horaceDied = false, fromBehind = Zombie:isBehind(attacker) })
        end
    end
    HoraceData.lastVehicleHitForce = hitForce
end

function HoraceVehicleDamage.DoHoraceCarDamage(Player, Vehicle, HitForce, HoraceType, HoraceDied, FromBehind)
    playServerSound("VehicleCrash", Vehicle:getSquare())

    local frontParts = {"EngineDoor", "Windshield", "WindowFrontLeft", "WindowFrontRight", "HeadlightLeft", "HeadlightRight", "TireFrontLeft", "TireFrontRight"}
    local backParts = {"TrunkDoor", "WindshieldRear", "WindowRearLeft", "WindowRearRight", "HeadlightRearLeft", "HeadlightRearRight", "TireRearLeft", "TireRearRight"}

    local engineDamage = HoraceVehicleDamage.GetHoraceEngineDamage(HoraceType, Vehicle, HitForce, HoraceDied)
    HoraceVehicleDamage.DamageVehiclePart(Vehicle, "Engine", engineDamage)
    Player:setHaloNote("Engine damage: " .. engineDamage, 255, 0, 0, 300)

    local batteryDamage = HoraceVehicleDamage.GetHoraceBatteryDamage(HoraceType, Vehicle, HitForce, HoraceDied)
    HoraceVehicleDamage.DamageVehiclePart(Vehicle, "Battery", batteryDamage)

    if FromBehind then
        for i, partName in ipairs(backParts) do
            local partDamage = HoraceVehicleDamage.GetHoracePartsDamage(HoraceType, Vehicle, partName, HitForce, HoraceDied)
            HoraceVehicleDamage.DamageVehiclePart(Vehicle, partName, partDamage)
        end
    else
        for i, partName in ipairs(frontParts) do
            local partDamage = HoraceVehicleDamage.GetHoracePartsDamage(HoraceType, Vehicle, partName, HitForce, HoraceDied)
            HoraceVehicleDamage.DamageVehiclePart(Vehicle, partName, partDamage)
        end
    end

    local shutoffChance = HoraceVehicleDamage.GetHoraceShutoffChance(HoraceType, Vehicle, HitForce, HoraceDied)
    if shutoffChance > 0 and ZombRand(shutoffChance) == 0 then
        Vehicle:shutOff()
        Vehicle:engineDoStalling()
    end
end

function HoraceVehicleDamage.GetHoraceEngineDamage(HoraceType, Vehicle, HitForce, HoraceDied)
    if HoraceType ~= "Pangolin" and HoraceType ~= "Crawler" and HoraceType ~= "Spider" and HoraceType ~= "Spiky" then
        return 0
    end
    local engineCondition = Vehicle:getPartById("Engine"):getCondition()
    return HoraceDied and engineCondition or ZombRand(HitForce * 10)
end

function HoraceVehicleDamage.GetHoraceBatteryDamage(HoraceType, Vehicle, HitForce, HoraceDied)
    if HoraceType ~= "Pangolin" and HoraceType ~= "Crawler" and HoraceType ~= "Spider" and HoraceType ~= "Spiky" then
        return 0
    end
    local scale = HoraceDied and 80 or 10
    return ZombRand(HitForce * scale)
end

function HoraceVehicleDamage.GetHoracePartsDamage(HoraceType, Vehicle, PartName, HitForce, HoraceDied)
    if HoraceType ~= "Pangolin" and HoraceType ~= "Crawler" and HoraceType ~= "Spider" and HoraceType ~= "Spiky" then
        return 0
    end
    local baseTireLightDamage = HoraceDied and 80 or 20
    if PartName == "TireFrontLeft" or PartName == "TireFrontRight" or PartName == "TireRearLeft" or PartName == "TireRearRight" then
        return baseTireLightDamage + ZombRand(math.floor(HitForce * 20))
    end
    if PartName == "HeadlightLeft" or PartName == "HeadlightRight" or PartName == "HeadlightRearLeft" or PartName == "HeadlightRearRight" then
        return baseTireLightDamage + ZombRand(math.floor(HitForce * 20))
    end
    if HoraceDied and PartName == "Windshield" or PartName == "WindshieldRear" then
        return 100
    end
    local baseDamage = HoraceDied and 80 or 20
    return baseDamage + ZombRand(math.floor(HitForce * 30))
end

function HoraceVehicleDamage.GetHoraceShutoffChance(HoraceType, Vehicle, HitForce, HoraceDied)
    if HoraceType ~= "Pangolin" and HoraceType ~= "Crawler" and HoraceType ~= "Spider" and HoraceType ~= "Spiky" then
        return 0
    end
    if HoraceDied or HitForce >= 1 then
        return 1
    else
        return 0
    end
end

function HoraceVehicleDamage.DamageVehiclePart(Vehicle, PartName, Damage)
    local part = PartName == "Battery" and Vehicle:getBattery() or Vehicle:getPartById(PartName)
    if part and (part:getInventoryItem() or PartName == "Engine") then
        local cond = part:getCondition()
        part:damage(Damage)

        if PartName == "HeadlightLeft" or PartName == "HeadlightRight" or PartName == "HeadlightRearLeft" or PartName == "HeadlightRearRight" then
            if part:getCondition() <= 0 then
                part:setInventoryItem(nil)
                Vehicle:transmitPartItem(part)
            end
        end
    end
end

function HoraceVehicleDamage.OnHoraceDead(Zombie, Horace)
    local attacker = Zombie:getAttackedBy()
    if Horace and attacker and attacker:isDriving() then
        sendClientCommand(attacker, 'horace', 'HoraceCrash', { vehicle = attacker:getVehicle():getId(), hitForce = 1, horace = Horace.Name, horaceDied = true, fromBehind = Zombie:isBehind(attacker) })
    end
end