require("Horace/HoraceTypes")
require("EiraZed/BannedOutfits")

BannedOutfits.BanItem("Base.Crawler")
BannedOutfits.BanItem("Base.Mutant")
BannedOutfits.BanItem("Base.Spider")
BannedOutfits.BanItem("Base.Pangolin")
BannedOutfits.BanItem("Base.Spiky")
BannedOutfits.BanItem("Base.Slurpy")
BannedOutfits.BanItem("Base.Scout")
BannedOutfits.BanItem("Base.Strongboy")

BannedOutfits.BanOutfit("Mutant_Costume")
BannedOutfits.BanOutfit("Crawler_Costume")
BannedOutfits.BanOutfit("Spider_Costume")
BannedOutfits.BanOutfit("Pangolin_Costume")
BannedOutfits.BanOutfit("Spiky_Costume")
BannedOutfits.BanOutfit("Slurpy_Costume")
BannedOutfits.BanOutfit("Scout_Costume")
BannedOutfits.BanOutfit("Strongboy_Costume")

local BreakFullPart = function(VehiclePart, HitForce)
    return VehiclePart:GetCondition()
end

local ScaledHitForceRandom = function(Scale, Bonus)
    Bonus = Bonus or 0
    return function(VehiclePart, HitForce)
        return Bonus + HitForce * Scale
    end
end

local KnoxVehicleCrashDamage = {
    GlobalParts = { "Engine", "Battery" },
    FrontParts = { "EngineDoor", "Windshield", "WindowFrontLeft", "WindowFrontRight", "HeadlightLeft", "HeadlightRight", "TireFrontLeft", "TireFrontRight" },
    BackParts = { "TrunkDoor", "WindshieldRear", "WindowRearLeft", "WindowRearRight", "HeadlightRearLeft", "HeadlightRearRight", "TireRearLeft", "TireRearRight" },
    EngineDamage = ScaledHitForceRandom(10),
    BatteryDamage = ScaledHitForceRandom(10),
    TireFrontLeftDamage = ScaledHitForceRandom(20, 20),
    TireFrontRightDamage = ScaledHitForceRandom(20, 20),
    TireRearLeftDamage = ScaledHitForceRandom(20, 20),
    TireRearRightDamage = ScaledHitForceRandom(20, 20),
    HeadlightLeftDamage = ScaledHitForceRandom(20, 20),
    HeadlightRightDamage = ScaledHitForceRandom(20, 20),
    HeadlightRearLeftDamage = ScaledHitForceRandom(20, 20),
    HeadlightRearRightDamage = ScaledHitForceRandom(20, 20),
    DefaultDamage = ScaledHitForceRandom(30, 20)
}

local KnoxVehicleKilledDamage = {
    GlobalParts = { "Engine", "Battery" },
    FrontParts = { "EngineDoor", "Windshield", "WindowFrontLeft", "WindowFrontRight", "HeadlightLeft", "HeadlightRight", "TireFrontLeft", "TireFrontRight" },
    BackParts = { "TrunkDoor", "WindshieldRear", "WindowRearLeft", "WindowRearRight", "HeadlightRearLeft", "HeadlightRearRight", "TireRearLeft", "TireRearRight" },
    EngineDamage = BreakFullPart,
    BatteryDamage = ScaledHitForceRandom(80),
    TireFrontLeftDamage = ScaledHitForceRandom(20, 80),
    TireFrontRightDamage = ScaledHitForceRandom(20, 80),
    TireRearLeftDamage = ScaledHitForceRandom(20, 80),
    TireRearRightDamage = ScaledHitForceRandom(20, 80),
    HeadlightLeftDamage = ScaledHitForceRandom(20, 80),
    HeadlightRightDamage = ScaledHitForceRandom(20, 80),
    HeadlightRearLeftDamage = ScaledHitForceRandom(20, 80),
    HeadlightRearRightDamage = ScaledHitForceRandom(20, 80),
    WindshieldDamage = BreakFullPart,
    WindshieldRearDamage = BreakFullPart,
    ShutoffChance = 1,
    DefaultDamage = ScaledHitForceRandom(30, 80)
}

HoraceTypes.Register("Crawler", "FullSuit", "Base.Crawler", {
    IdleSound = "CrawlerClick",
    ChargeSound = "CrawlerCombat",
    BumpOnCharge = {
        Cooldown = 8
    },
    Retreating = {
        Chance = 1 / 5,
        TimeSpentRetreating = 5,
        ReturnAfter = true,
        ResetOnHit = true,
        RetreatResetsBumpOnCharge = true,
        LimitOnce = true,
        MinRange = 5,
        MaxRange = 10
    },
    RollOnGround = {
        Chance = 1 / 3
    },
    VehicleCrashed = KnoxVehicleCrashDamage,
    VehicleKilled = KnoxVehicleKilledDamage,
    AvoidDamageChance = 1 - 1 / 9
})

HoraceTypes.Register("Spider", "FullSuit", "Base.Spider", {
    IdleSound = "SpiderChirp",
    ChargeSound = "SpiderCombat",
    VehicleCrashed = KnoxVehicleCrashDamage,
    VehicleKilled = KnoxVehicleKilledDamage,
    RollOnGround = {
        Chance = 1 / 2
    },
    AvoidDamageChance = 1 - 1 / 8
})

HoraceTypes.Register("Pangolin", "FullSuit", "Base.Pangolin", {
    IdleSound = "PangolinGrowl",
    ChargeSound = "PangolinCombat",
    AvoidCrawling = true,
    VehicleCrashed = KnoxVehicleCrashDamage,
    VehicleKilled = KnoxVehicleKilledDamage,
    TripStompers = {
        Chance = 2 / 3
    },
    AvoidDamageChance = 1 - 1 / 10
})

HoraceTypes.Register("Spiky", "FullSuit", "Base.Spiky", {
    IdleSound = "SpikyIdle",
    ChargeSound = "SpikyCombat",
    VehicleCrashed = KnoxVehicleCrashDamage,
    VehicleKilled = KnoxVehicleKilledDamage,
    InjuryInflicting = {
        Injury = "DeepWound",
        OrderedPartChoices = {{
            "Hand_L",
            "Hand_R",
            "ForeArm_L",
            "ForeArm_R",
            "UpperArm_L",
            "UpperArm_R"
        }, {
            "Torso_Upper",
            "Torso_Lower"
        }, {
            "UpperLeg_L",
            "UpperLeg_R",
            "LowerLeg_L",
            "LowerLeg_R",
            "Foot_L",
            "Foot_R"
        }, {
            "Hand_L",
            "Hand_R",
            "ForeArm_L",
            "ForeArm_R",
            "UpperArm_L",
            "UpperArm_R",
            "Torso_Upper",
            "Torso_Lower",
            "UpperLeg_L",
            "UpperLeg_R",
            "LowerLeg_L",
            "LowerLeg_R",
            "Foot_L",
            "Foot_R"
         }}
    },
    RollOnGround = {
        Chance = 1 / 5
    },
    AvoidDamageChance = 1 - 1 / 6
})

HoraceTypes.Register("Slurpy", "FullSuit", "Base.Slurpy", {
    IdleSound = "SlurpyIdle",
    ChargeSound = "SlurpyCharge",
    BumpOnCharge = {
        Cooldown = 8,
        CooldownOnParry = true
    },
    Retreating = {
        Chance = 1 / 2,
        ReturnAfterSeconds = 5,
        LimitOnce = true,
        MinRange = 5,
        MaxRange = 10
    },
    Slurping = {
        Sound = "SlurpySlurp",
        Cooldown = 5,
        PartChoices = { "Neck" },
        Neck = {
            Bite = true,
            Bleeding = true,
            MinDamage = 0.1,
            MaxDamage = 0.2,
            Message = "*latches onto %target%'s neck*"
        },
        AvoidKneeling = true
    },
    RollOnGround = {
        Chance = 1 / 3
    },
    AvoidDamageChance = 1 - 1 / 3
})

HoraceTypes.Register("Scout", "FullSuit", "Base.Scout", {
    Retreating = {
        Chance = 1,
        TimeSpentRetreating = 5,
        MinRange = 10,
        MaxRange = 20
    },
    Screeching = {
        TriggerChance = 1,
        ChargeTime = 10,
        ScreechTime = 10,
        PreparationSound = "ScoutInhale",
        Sound = "ScoutScream",
        SoundVolume = 60,
        SoundRange = 60,
        Cooldown = 30,
        RetreatAfter = true,
        ResetsOnHit = true,
        WarnMessage = "*takes a deep breath and prepares to screech*",
        ScreechMessage = "*produces a shrilling scream summoning nearby creatures*",
        NeverAttacks = true,
    },
    AvoidDamageChance = 1 - 1 / 3,
    AvoidCrawling = true
})

HoraceTypes.Register("Strongboy", "FullSuit", "Base.Strongboy", {
    InjuryInflicting = {
        Injury = "Fracture",
        OrderedPartChoices = {{
            "Hand_L",
            "Hand_R",
            "ForeArm_L",
            "ForeArm_R",
            "UpperArm_L",
            "UpperArm_R"
        }, {
            "Torso_Upper",
            "Torso_Lower"
        }, {
            "UpperLeg_L",
            "UpperLeg_R",
            "LowerLeg_L",
            "LowerLeg_R",
            "Foot_L",
            "Foot_R"
        }, {
            "Hand_L",
            "Hand_R",
            "ForeArm_L",
            "ForeArm_R",
            "UpperArm_L",
            "UpperArm_R",
            "Torso_Upper",
            "Torso_Lower",
            "UpperLeg_L",
            "UpperLeg_R",
            "LowerLeg_L",
            "LowerLeg_R",
            "Foot_L",
            "Foot_R"
         }}
    },
    VehicleCrashed = KnoxVehicleCrashDamage,
    VehicleKilled = KnoxVehicleKilledDamage,
    AvoidDamageChance = 1 - 1 / 8
})

HoraceTypes.Register("Mimic", "FullSuit", "Base.Mimic", {
    Mimic = {
        MimicOnCharge = true,
        ActiveMimicSlot = "Neck",
        ActiveMimicItem = "Base.MimicActive"
    },
    IdleSound = "MimicIdle",
    AvoidDamageChance = 1 - 1 / 4
})

HoraceTypes.Register("MimicActive", "Neck", "Base.MimicActive", {
    Mimic = {
        Active = true,
        InactiveMimicSlot = "FullSuit",
        InactiveMimicItem = "Base.Mimic"
    },
    Retreating = {
        Chance = 1 / 5,
        TimeSpentRetreating = 5,
        ReturnAfter = true,
        LimitOnce = true,
        MinRange = 8,
        MaxRange = 16
    },
    AvoidDamageChance = 1 - 1 / 4
})